'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _browserConfig = require('./browser-config.json');

var _browserConfig2 = _interopRequireDefault(_browserConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// exclude legacy and server config
delete _browserConfig2.default.legacy;
delete _browserConfig2.default.server;

var checkMutability = true;

/**
 * Underlying get mechanism
 *
 * @private
 * @method getImpl
 * @param object {object} - Object to get the property for
 * @param property {string | array[string]} - The property name to get (as an array or '.' delimited string)
 * @return value {*} - Property value, including undefined if not defined.
 */
var getImpl = function getImpl(object, property) {
  var elems = Array.isArray(property) ? property : property.split('.');
  var name = elems[0];
  var value = object[name];
  if (elems.length <= 1) {
    return value;
  }
  // Note that typeof null === 'object'
  if (value === null || (typeof value === 'undefined' ? 'undefined' : _typeof(value)) !== 'object') {
    return undefined;
  }
  return getImpl(value, elems.slice(1));
};

var isObject = function isObject(obj) {
  return obj !== null && (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && !Array.isArray(obj);
};

var makeImmutable = function makeImmutable(object, property, value) {
  var properties = null;

  // Backwards compatibility mode where property/value can be specified
  if (typeof property === 'string') {
    return Object.defineProperty(object, property, {
      value: typeof value === 'undefined' ? object[property] : value,
      writable: false,
      configurable: false
    });
  }

  // Get the list of properties to work with
  if (Array.isArray(property)) {
    properties = property;
  } else {
    properties = Object.keys(object);
  }

  // Process each property
  for (var i = 0; i < properties.length; i++) {
    var propertyName = properties[i];
    var _value = object[propertyName];

    Object.defineProperty(object, propertyName, {
      value: _value,
      writable: false,
      configurable: false
    });

    // Call recursively if an object.
    if (isObject(_value)) {
      makeImmutable(_value);
    }
  }

  return object;
};

exports.default = _extends({}, _browserConfig2.default, {
  get: function get(property) {
    if (property === null || property === undefined) {
      throw new Error("Calling config.get with null or undefined argument");
    }
    var value = getImpl(_browserConfig2.default, property);

    // Produce an exception if the property doesn't exist
    if (value === undefined) {
      throw new Error('Configuration property "' + property + '" is not defined');
    }

    // Make configurations immutable after first get (unless disabled)
    if (checkMutability) {
      if (!(process.env.ALLOW_CONFIG_MUTATIONS || false)) {
        makeImmutable(_browserConfig2.default);
      }
      checkMutability = false;
    }

    // Return the value
    return value;
  },
  has: function has(property) {
    // While get() throws an exception for undefined input, has() is designed to test validity, so false is appropriate
    if (property === null || property === undefined) {
      return false;
    }
    return getImpl(_browserConfig2.default, property) !== undefined;
  }
});
//# sourceMappingURL=browser.js.map